package com.argotec.siar.model;

import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.util.List;

import static com.argotec.siar.util.FormatUtil.toDate;
import static com.argotec.siar.util.FormatUtil.toFloat;

@Entity
@Table(name = "variable")
@Data
@NoArgsConstructor
public class Variable {

    private static final String GENERATOR_NAME = "variable";

    @Id
    @GenericGenerator(
            name = GENERATOR_NAME,
            strategy = "org.hibernate.id.enhanced.SequenceStyleGenerator"
            /*parameters = {
                    @Parameter(name = "sequence_name", value = "SQ-" + GENERATOR_NAME),
                    @Parameter(name = "initial_value", value = "1"),
                    @Parameter(name = "increment_size", value = "1")
            }*/
    )
    @GeneratedValue(generator = GENERATOR_NAME)
    private Long id;

    private LocalDate date;

    private float kc;
    private float eto;
    private float etc;
    private float pe;
    private float etcMinusPe;

    @NotNull
    @Column(name = "crop_id")
    private Long cropId;

    public Variable(Long cropId, List<String> tableRow) {
        this.cropId = cropId;
        parseTableRow(tableRow);
    }

    private void parseTableRow(List<String> tableRow) {
        date = toDate(tableRow.get(0));

        kc = toFloat(tableRow.get(1));
        eto = toFloat(tableRow.get(2));
        etc = toFloat(tableRow.get(3));
        pe = toFloat(tableRow.get(4));
        etcMinusPe = toFloat(tableRow.get(5));
    }
}
