package com.argotec.siar.scraper;

import com.argotec.scraper.Scraper;
import com.argotec.scraper.ScraperDataSource;
import com.argotec.siar.model.Crop;
import com.argotec.siar.model.Station;
import com.argotec.siar.util.ScrapingUrl;
import lombok.extern.slf4j.Slf4j;
import org.openqa.selenium.By;

import java.util.List;
import java.util.stream.Collectors;

import static com.argotec.scraper.Scraper.source;
import static com.argotec.siar.scraper.ScraperVariables.*;

@Slf4j
public class ScraperCrops implements ScraperDataSource<Crop> {

    private final List<Crop> existingCrops;
    private final Station station;

    public ScraperCrops(List<Crop> existingCrops, Station station) {
        this.existingCrops = existingCrops;
        this.station = station;
    }

    @Override
    public List<Crop> scrapeAll() {
        log.info("Fetching crops of station '{}' using WebScrapper", station.getName());

        try (Scraper scraper = source(ScrapingUrl.getUrlNn)) {
            scraper.doAction(selectAuCommunityAction(station.getProvince().getAutonomousCommunity().getName()))
                    .doAction(selectProvinceAction(station.getProvince().getName()))
                    .doAction(selectStationAction(station.getName()));

            return parseCropsFromWeb(scraper);
        }
    }

    private List<Crop> parseCropsFromWeb(Scraper scraper) {
        return scraper.findRootElement(By.id("ContentPlaceHolder1_cboCultivo"))
                .toSelectElement()
                .getOptions()
                .stream()
                .map(op -> {
                    String cropName = op.getText();
                    log.info("Parsing station {} crop {}", station.getName(), cropName);
                    Crop crop = getCropFromList(cropName);
                    if (crop == null) {
                        log.info("Creating a new crop {}", cropName);
                        crop = new Crop();
                        crop.setName(cropName);
                        existingCrops.add(crop);
                    }
                    return crop;
                })
                .collect(Collectors.toList());
    }

    private Crop getCropFromList(String cropName) {
        return existingCrops.stream().filter(c -> c.getName().equals(cropName)).findFirst().orElse(null);
    }
}
