package com.argotec.siar.scraper;

import com.argotec.scraper.Function;
import com.argotec.scraper.Scraper;
import com.argotec.scraper.ScraperDataSource;
import com.argotec.siar.model.Measurement;
import com.argotec.siar.model.Station;
import com.argotec.siar.util.ScrapingUrl;
import lombok.extern.slf4j.Slf4j;
import org.openqa.selenium.By;

import java.time.LocalDate;
import java.util.List;
import java.util.stream.Collectors;

import static com.argotec.scraper.Scraper.source;
import static com.argotec.siar.scraper.ScraperCommunities.selectAuCommunityAction;
import static com.argotec.siar.scraper.ScraperProvinces.selectProvinceAction;
import static com.argotec.siar.scraper.ScraperStations.addStation;
import static com.argotec.siar.scraper.SiarScraperUtil.setEndDate;
import static com.argotec.siar.scraper.SiarScraperUtil.setStartDate;

@Slf4j
public class ScraperMeasurements implements ScraperDataSource<Measurement> {

    private final Station station;
    private final LocalDate startDate;
    private final LocalDate endDate;

    public ScraperMeasurements(Station station, LocalDate startDate, LocalDate endDate) {
        this.station = station;
        this.startDate = startDate;
        this.endDate = endDate;
    }

    @Override
    public List<Measurement> scrapeAll() {
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {

            scraper.doAction(selectAuCommunityAction(station.getProvince().getAutonomousCommunity().getName()))
                    .doAction(selectProvinceAction(station.getProvince().getName()))
                    .doAction(addStation(station.getName()))
                    .doAction(setStartDate(startDate))
                    .doAction(setEndDate(endDate));

            scraper.findRootElement(By.id("ContentPlaceHolder1_btnConsultar"))
                    .click()
                    .and()
                    .waitSeconds(2)
                    .switchTo()
                    .waitSeconds(2);

            return toMeasurements(station, Function.createTable(scraper.getWebDriver()));
        }
    }

    private List<Measurement> toMeasurements(Station station, List<List<String>> table) {
        return table.stream()
                .skip(1)
                .map(row -> new Measurement(station.getId(), row))
                .collect(Collectors.toList());
    }
}
