package com.argotec.siar.scraper;

import com.argotec.scraper.Scraper;
import com.argotec.scraper.ScraperAction;
import com.argotec.scraper.ScraperDataSource;
import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.model.Province;
import com.argotec.siar.util.ScrapingUrl;
import lombok.extern.slf4j.Slf4j;
import org.openqa.selenium.By;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ISelect;

import java.util.List;
import java.util.stream.Collectors;

import static com.argotec.scraper.Scraper.source;
import static com.argotec.scraper.ScraperUtil.findOption;
import static com.argotec.scraper.ScraperUtil.selectOption;
import static com.argotec.siar.scraper.ScraperCommunities.selectAuCommunityAction;

@Slf4j
public class ScraperProvinces implements ScraperDataSource<Province> {

    private final static String SELECT_ID = "DropDownListProvincia";

    private final AutonomousCommunity autonomousCommunity;

    public ScraperProvinces(AutonomousCommunity autonomousCommunity) {
        this.autonomousCommunity = autonomousCommunity;
    }

    @Override
    public List<Province> scrapeAll() {
        log.info("Fetching provinces of {} using WebScrapper", autonomousCommunity.getName());
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {
            scraper.doAction(selectAuCommunityAction(autonomousCommunity.getName()));
            return findSelect(scraper)
                    .getOptions()
                    .stream()
                    .map(this::createEntityFromOption)
                    .collect(Collectors.toList());
        }
    }

    public Province scrapeOne(String name) {
        log.info("Fetching province {} of {} using WebScrapper", name, autonomousCommunity.getName());
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {
            scraper.doAction(selectAuCommunityAction(autonomousCommunity.getName()));

            WebElement optionEl = findOption(findSelect(scraper), name);
            if (optionEl != null) {
                return createEntityFromOption(optionEl);
            }
            return null;
        }
    }

    private ISelect findSelect(Scraper scraper) {
        return scraper.findRootElement(By.id(SELECT_ID)).toSelectElement();
    }

    private Province createEntityFromOption(WebElement option) {
        return new Province(option.getText(), autonomousCommunity);
    }

    static ScraperAction selectProvinceAction(String province) {
        String onErrorMessage = "Error: Incorrectly introduced province";
        return scraper -> {
            scraper.findRootElement(By.id("ContentPlaceHolder1_RadioButtonProvincias")).click();
            selectOption(SELECT_ID, province, onErrorMessage).doAction(scraper);
        };
    }
}
