package com.argotec.siar.service.impl;

import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.repository.AutonomousCommunityRepository;
import com.argotec.siar.scraper.ScraperCommunities;
import com.argotec.siar.service.AutonomousCommunityService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

import static com.argotec.siar.util.FormatUtil.stringsAreEqualAfterNormalization;

@Service
@Slf4j
public class AutonomousCommunityServiceImpl implements AutonomousCommunityService {

    private final AutonomousCommunityRepository autonomousCommunityRepository;

    public AutonomousCommunityServiceImpl(AutonomousCommunityRepository autonomousCommunityRepository) {
        this.autonomousCommunityRepository = autonomousCommunityRepository;
    }

    @Override
    public List<AutonomousCommunity> getAllCommunities(boolean ws) {
        List<AutonomousCommunity> list = autonomousCommunityRepository.findAll();

        if (ws || list.isEmpty()) {
            list = fetchCommunitiesUsingWebScrapping();
        }

        return list;
    }

    @Override
    public AutonomousCommunity getByName(String name) {
        AutonomousCommunity autonomousCommunity = autonomousCommunityRepository.findAll().stream()
                .filter(aC -> stringsAreEqualAfterNormalization(aC.getName(), name))
                .findFirst()
                .orElse(null);

        if (autonomousCommunity == null) {
            autonomousCommunity = getScrapper().scrapeOne(name);
            if (autonomousCommunity != null) {
                autonomousCommunityRepository.save(autonomousCommunity);
            }
        }
        return autonomousCommunity;
    }

    private List<AutonomousCommunity> fetchCommunitiesUsingWebScrapping() {
        List<AutonomousCommunity> list = getScrapper().scrapeAll();

        list.stream()
                .filter(c -> autonomousCommunityRepository.findByName(c.getName()) == null)
                .peek(c -> log.info("Saving {} community in the repository", c.getName()))
                .forEach(autonomousCommunityRepository::save);

        return list;
    }

    private ScraperCommunities getScrapper() {
        return new ScraperCommunities();
    }
}
