package com.argotec.siar.service.impl;

import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.model.IrrigationCommunity;
import com.argotec.siar.repository.IrrigationCommunityRepository;
import com.argotec.siar.scraper.ScraperIrrigationCommunities;
import com.argotec.siar.service.AutonomousCommunityService;
import com.argotec.siar.service.IrrigationCommunityService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

import static com.argotec.siar.util.FormatUtil.stringsAreEqualAfterNormalization;

@Service
@Slf4j
public class IrrigationCommunityServiceImpl implements IrrigationCommunityService {

    private final IrrigationCommunityRepository irrigationCommunityRepository;

    private final AutonomousCommunityService autonomousCommunityService;

    public IrrigationCommunityServiceImpl(IrrigationCommunityRepository irrigationCommunityRepository, AutonomousCommunityService autonomousCommunityService) {
        this.irrigationCommunityRepository = irrigationCommunityRepository;
        this.autonomousCommunityService = autonomousCommunityService;
    }

    @Override
    public List<IrrigationCommunity> getAllIrrCommunities() {
        return irrigationCommunityRepository.findAll();
    }

    @Override
    public List<IrrigationCommunity> getAllIrrCommunitiesByAuComm(String autonomousCommunityName, boolean ws) {
        AutonomousCommunity autonomousCommunity = autonomousCommunityService.getByName(autonomousCommunityName);
        List<IrrigationCommunity> list = autonomousCommunity.getIrrigationCommunities();

        if ((ws || list.isEmpty()) && autonomousCommunityName != null) {
            list = fetchIrrigationCommunitiesUsingWebScrapping(autonomousCommunityName);
        }

        return list;
    }

    @Override
    public IrrigationCommunity getByName(String irrigationCommunityName, String autonomousCommunityName, boolean ws) {
        IrrigationCommunity irrigationCommunity = irrigationCommunityRepository.findAll().stream()
                .filter(iC -> stringsAreEqualAfterNormalization(iC.getName(), irrigationCommunityName))
                .findFirst()
                .orElse(null);

        if ((ws || irrigationCommunity == null) && autonomousCommunityName != null) {
            irrigationCommunity = getScrapper(autonomousCommunityName).scrapeOne(irrigationCommunityName);
            if (irrigationCommunity != null) {
                irrigationCommunityRepository.save(irrigationCommunity);
            }
        }

        return irrigationCommunity;
    }

    private List<IrrigationCommunity> fetchIrrigationCommunitiesUsingWebScrapping(String autonomousCommunityName) {
        List<IrrigationCommunity> list = getScrapper(autonomousCommunityName).scrapeAll();

        list.stream()
                .filter(p -> irrigationCommunityRepository.findByName(p.getName()) == null)
                .peek(p -> log.info("Saving {} IrrigationCommunity in the repository", p.getName()))
                .forEach(irrigationCommunityRepository::save);

        return list;
    }

    private ScraperIrrigationCommunities getScrapper(String autonomousCommunityName) {
        AutonomousCommunity autonomousCommunity = autonomousCommunityService.getByName(autonomousCommunityName);
        return new ScraperIrrigationCommunities(autonomousCommunity);
    }
}
