package com.argotec.siar.service.impl;

import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.model.Province;
import com.argotec.siar.repository.ProvinceRepository;
import com.argotec.siar.scraper.ScraperProvinces;
import com.argotec.siar.service.AutonomousCommunityService;
import com.argotec.siar.service.ProvinceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

import static com.argotec.siar.util.FormatUtil.stringsAreEqualAfterNormalization;

@Service
@Slf4j
public class ProvinceServiceImpl implements ProvinceService {

    private final ProvinceRepository provinceRepository;

    private final AutonomousCommunityService autonomousCommunityService;

    public ProvinceServiceImpl(ProvinceRepository provinceRepository, AutonomousCommunityService autonomousCommunityService) {
        this.provinceRepository = provinceRepository;
        this.autonomousCommunityService = autonomousCommunityService;
    }

    @Override
    public List<Province> getAllProvinces() {
        return provinceRepository.findAll();
    }

    @Override
    public List<Province> getAllProvincesByAuComm(String autonomousCommunityName, boolean ws) {
        AutonomousCommunity autonomousCommunity = autonomousCommunityService.getByName(autonomousCommunityName);
        List<Province> list = autonomousCommunity.getProvinces();

        if ((ws || list.isEmpty()) && autonomousCommunityName != null) {
            list = fetchProvincesUsingWebScrapping(autonomousCommunityName);
        }

        return list;
    }

    @Override
    public Province getByName(String provinceName, String communityName) {
        Province province = provinceRepository.findAll().stream()
                .filter(p -> stringsAreEqualAfterNormalization(p.getName(), provinceName))
                .findFirst()
                .orElse(null);

        if (province == null && communityName != null) {
            province = getScrapper(communityName).scrapeOne(provinceName);
            if (province != null) {
                provinceRepository.save(province);
            }
        }

        return province;
    }

    private List<Province> fetchProvincesUsingWebScrapping(String autonomousCommunityName) {
        List<Province> list = getScrapper(autonomousCommunityName).scrapeAll();

        list.stream()
                .filter(p -> provinceRepository.findByName(p.getName()) == null)
                .peek(p -> log.info("Saving {} province in the repository", p.getName()))
                .forEach(provinceRepository::save);

        return list;
    }

    private ScraperProvinces getScrapper(String autonomousCommunityName) {
        AutonomousCommunity autonomousCommunity = autonomousCommunityService.getByName(autonomousCommunityName);
        return new ScraperProvinces(autonomousCommunity);
    }
}
