package com.argotec.siar.model;

import com.argotec.siar.TestApplication;
import com.google.common.collect.Sets;
import org.assertj.core.util.Lists;
import org.hamcrest.Matcher;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.autoconfigure.orm.jpa.TestEntityManager;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.List;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.junit.Assert.assertTrue;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = TestApplication.class)
@AutoConfigureTestDatabase
@DataJpaTest
public class AutonomousCommunityTest {

    @Autowired
    private TestEntityManager entityManager;

    @Test
    public void shouldSaveCommunityCorrectly() {
        AutonomousCommunity au = saveAutonomyCommunity("name");
        assertThatProvincesAre(au, hasSize(0));
    }

    @Test
    public void shouldSetProvincesCorrectly() {
        AutonomousCommunity au = saveAutonomyCommunity("name");
        List<Province> provinces = Lists.newArrayList(
                new Province("name1", au),
                new Province("name2", au)
        );

        au.setProvinces(provinces);

        au = entityManager.persistFlushFind(au);

        assertThatProvincesAre(au, equalTo(provinces));
    }

    @Test
    public void shouldAddProvinceCorrectly() {
        AutonomousCommunity au = saveAutonomyCommunity("name");
        Province province = new Province("name3", au);

        assertTrue(au.addProvince(province));

        au = entityManager.find(AutonomousCommunity.class, au.getId());

        assertThatProvincesAre(au, equalTo(Sets.newHashSet(province)));
    }

    @Test
    public void shouldRemoveProvinceCorrectly() {
        AutonomousCommunity au = saveAutonomyCommunity("name");
        Province province1 = new Province("name1", au);
        Province province2 = new Province("name2", au);

        au.setProvinces(Lists.newArrayList(province1, province2));

        entityManager.flush();

        au = entityManager.find(AutonomousCommunity.class, au.getId());
        au.removeProvince(province1);

        au = entityManager.persistFlushFind(au);

        assertThatProvincesAre(au, equalTo(Sets.newHashSet(province2)));
    }

    private AutonomousCommunity saveAutonomyCommunity(String name) {
        AutonomousCommunity autonomousCommunity = new AutonomousCommunity(name);
        return entityManager.persist(autonomousCommunity);
    }

    private void assertThatProvincesAre(AutonomousCommunity autonomousCommunity, Matcher provincesMatcher) {
        assertThat(autonomousCommunity, allOf(
                hasProperty("id", equalTo(autonomousCommunity.getId())),
                hasProperty("name", equalTo(autonomousCommunity.getName())),
                hasProperty("provinces", provincesMatcher)
        ));
    }
}