package com.argotec.siar.scraper;

import com.argotec.scraper.Scraper;
import com.argotec.scraper.ScraperDataSource;
import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.model.Province;
import com.argotec.siar.model.Station;
import com.argotec.siar.util.ScrapingUrl;
import org.junit.Test;
import org.openqa.selenium.By;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ISelect;

import java.util.Collections;
import java.util.List;

import static com.argotec.scraper.Scraper.source;
import static org.hamcrest.Matchers.*;
import static org.junit.Assert.*;

public class ScraperStationsTest {

    @Test
    public void shouldRetrieveOptionsFromStationFieldAndMappedToModelEntity() {
        Province province = new Province("Valladolid", new AutonomousCommunity("Castilla y León"));
        ScraperDataSource<Station> scraperStation = new ScraperStations(Collections.emptyList(), "Castilla y León", province);

        List<Station> stations = scraperStation.scrapeAll();

        assertFalse(stations.isEmpty());
        assertThat(stations, hasItem(hasProperty("name", equalTo("Olmedo"))));
    }

    @Test
    public void shouldAddSuccessfullyStationsToStationBox() {
        Scraper scraper = getScraper();

        ScraperStations.addStation("Adra").doAction(scraper);
        ScraperStations.addStation("Fiñana").doAction(scraper);

        List<WebElement> options = fetchStationSelectedAreaEl(scraper)
                .getOptions();

        assertThat(options, contains(
                hasProperty("text", equalTo("Adra")),
                hasProperty("text", equalTo("Fiñana"))
        ));
    }

    @Test
    public void shouldRaiseIllegalArgumentException_whenSelectedStationDoesNotExist() {
        Scraper scraper = getScraper();

        try {
            ScraperStations.addStation("SOMETHING").doAction(scraper);
            fail("Has to throw IllegalArgumentException");
        } catch (IllegalArgumentException ex) {
            assertThat(ex.getMessage(), equalTo("ERROR: SOMETHING - Incorrectly introduced station"));
        }
    }

    private Scraper getScraper() {
        return source(ScrapingUrl.getUrlDq);
    }

    private ISelect fetchStationSelectedAreaEl(Scraper scraper) {
        return scraper.findRootElement(By.id("ListBoxEstaciones")).toSelectElement();
    }
}