package com.argotec.restapi.controller;

import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.model.IrrigationCommunity;
import com.argotec.siar.model.Province;
import com.argotec.siar.service.AutonomousCommunityService;
import com.argotec.siar.service.IrrigationCommunityService;
import com.argotec.siar.service.ProvinceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/v1/communities")
@Api(tags = {"Autonomous Community"})
public class AutonomousCommunityController {

    private final AutonomousCommunityService autonomousCommunityService;
    private final ProvinceService provinceService;
    private final IrrigationCommunityService irrigationCommunityService;

    public AutonomousCommunityController(AutonomousCommunityService autonomousCommunityService, ProvinceService provinceService, IrrigationCommunityService irrigationCommunityService) {
        this.autonomousCommunityService = autonomousCommunityService;
        this.provinceService = provinceService;
        this.irrigationCommunityService = irrigationCommunityService;
    }

    @GetMapping
    @ApiOperation("Get all communities")
    public List<AutonomousCommunity> getAllCommunities(@RequestParam(required = false) boolean ws) {
        return autonomousCommunityService.getAllCommunities(ws);
    }

    @GetMapping("/{name}")
    @ApiOperation("Get a community by name")
    public AutonomousCommunity getCommunityByName(@PathVariable String name) {
        return autonomousCommunityService.getByName(name);
    }

    @GetMapping("/{name}/provinces")
    @ApiOperation("Get all provinces that belongs to a specific community")
    public List<Province> getProvincesOfACommunity(@PathVariable String name,
                                                   @RequestParam(required = false) boolean ws) {
        return provinceService.getAllProvincesByAuComm(name, ws);
    }

    @GetMapping("/{name}/irrigation-communities")
    @ApiOperation("Get all irrigation communities that belongs to a specific community")
    public List<IrrigationCommunity> getIrrigationCommuntiesOfACommunity(@PathVariable String name,
                                                                         @RequestParam(required = false) boolean ws) {
        return irrigationCommunityService.getAllIrrCommunitiesByAuComm(name, ws);
    }
}
