package com.argotec.restapi.controller;

import com.argotec.siar.model.Province;
import com.argotec.siar.model.Station;
import com.argotec.siar.service.ProvinceService;
import com.argotec.siar.service.StationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/v1/provinces")
@Api(tags = {"Province"})
public class ProvinceController {

    private final ProvinceService provinceService;
    private final StationService stationService;

    public ProvinceController(ProvinceService provinceService, StationService stationService) {
        this.provinceService = provinceService;
        this.stationService = stationService;
    }

    @GetMapping
    @ApiOperation("Get all provinces")
    public List<Province> getAllProvinces(@RequestParam(required = false) String communityName,
                                          @RequestParam(required = false) boolean ws) {
        return communityName == null
                ? provinceService.getAllProvinces()
                : provinceService.getAllProvincesByAuComm(communityName, ws);
    }

    @GetMapping("/{name}")
    @ApiOperation("Get a province by name")
    public Province getProvinceByName(@PathVariable String name,
                                      @RequestParam(required = false) String communityName) {
        return provinceService.getByName(name, communityName);
    }

    @GetMapping("/{name}/stations")
    @ApiOperation("Get all stations that belongs to a specific province")
    public List<Station> getStationsOfAProvince(@PathVariable String name,
                                                @RequestParam(required = false) String communityName,
                                                @RequestParam(required = false) boolean ws) {
        Province province = provinceService.getByName(name, communityName);
        return stationService.getAllStations(province, ws);
    }
}
