package com.argotec.restapi.controller;

import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.time.format.DateTimeParseException;

@RestControllerAdvice
@Slf4j
public class RestExceptionHandler {

    @ExceptionHandler(DateTimeParseException.class)
    public ErrorResponse handleDateTimeParseException(DateTimeParseException ex) {
        return new ErrorResponse(String.format("Invalid provided date (%s)", ex.getParsedString()));
    }

    @ExceptionHandler(RuntimeException.class)
    public ErrorResponse handleRuntimeException(RuntimeException ex) {
        log.error("Handled RuntimeException", ex);
        return new ErrorResponse(ex.getMessage());
    }

    @Getter
    @RequiredArgsConstructor
    private class ErrorResponse {
        private final String error;
    }
}
