package com.argotec.restapi.controller;

import com.argotec.siar.model.Crop;
import com.argotec.siar.model.Measurement;
import com.argotec.siar.model.Station;
import com.argotec.siar.model.Variable;
import com.argotec.siar.service.MeasurementService;
import com.argotec.siar.service.StationService;
import com.argotec.siar.service.VariableService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Set;

import static com.argotec.restapi.controller.Util.toLocalDate;

@RestController
@RequestMapping("/v1/stations")
@Api(tags = {"Station"})
public class StationController {

    private final StationService stationService;
    private final MeasurementService measurementService;
    private final VariableService variableService;

    public StationController(StationService stationService, MeasurementService measurementService, VariableService variableService) {
        this.stationService = stationService;
        this.measurementService = measurementService;
        this.variableService = variableService;
    }

    @GetMapping
    @ApiOperation("Get all stations")
    public List<Station> getAllStations(@RequestParam(required = false) boolean ws) {
        return stationService.getAllStations(ws);
    }

    @GetMapping("/{name}")
    @ApiOperation("Get a station by name")
    public Station getStationByName(@PathVariable String name) {
        return stationService.getByName(name);
    }

    @GetMapping("/{name}/measurements")
    @ApiOperation("Get all measurements that belongs to a specific station")
    public List<Measurement> getMeasurementsOfAStation(@PathVariable String name,
                                                       @RequestParam String startDate,
                                                       @RequestParam String endDate) {
        Station station = stationService.getByName(name);
        return measurementService.getAllByStationAndDateBetween(station, toLocalDate(startDate), toLocalDate(endDate));
    }

    @GetMapping("/{name}/crops")
    @ApiOperation("Get all crops that belongs to a specific station")
    public Set<Crop> getCropsOfAStation(@PathVariable String name,
                                        @RequestParam(required = false) boolean ws) {
        Station station = stationService.getByName(name);
        return stationService.getCropsByStation(station, ws);
    }

    @GetMapping("/{name}/crops/{cropName}/variables")
    @ApiOperation("Get all variables that belongs to a specific station and crop")
    public List<Variable> getCropsOfAStationAndCrop(@PathVariable String name,
                                                    @PathVariable String cropName,
                                                    @RequestParam String startDate,
                                                    @RequestParam String endDate,
                                                    @RequestParam(required = false) boolean ws) {
        Station station = stationService.getByName(name);
        return variableService.getAllByStationAndCropAndDateBetween(station, cropName, toLocalDate(startDate), toLocalDate(endDate), ws);
    }
}
