package com.argotec.restapi.controller;

import com.argotec.siar.service.impl.HomeService;
import org.assertj.core.api.Assertions;
import org.hamcrest.MatcherAssert;
import org.hamcrest.Matchers;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.junit4.SpringRunner;

@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class HomeControllerTest {

    private static final String BASE_URL = "http://localhost:";

    @LocalServerPort
    private int port;

    @Autowired
    private TestRestTemplate restTemplate;

    private static final String MESSAGE = "Welcome to your API REST to get meteorological data";

    private HomeController homeController;
    private HomeService homeService;

    @Before
    public void setUp() {
        homeService = Mockito.mock(HomeService.class);
        homeController = new HomeController(homeService);
    }

    @Test
    public void shouldReturnMessageAndValueWith200StatusCode() {
        Mockito.when(homeService.mWelcome()).thenReturn(MESSAGE);
        ResponseEntity httpResponse = homeController.messageWelcome();
        MatcherAssert.assertThat(httpResponse.getStatusCode(), Matchers.is(HttpStatus.OK));
        MatcherAssert.assertThat(MESSAGE, Matchers.is(httpResponse.getBody()));
    }

    @Test
    public void shouldReturnWelcomeMessage() {
        Assertions.assertThat(this.restTemplate.getForObject(BASE_URL + port + "/", String.class)).contains(MESSAGE);
    }
}


