package com.argotec.siar.service;

import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.repository.AutonomousCommunityRepository;
import com.argotec.siar.service.impl.AutonomousCommunityServiceImpl;
import org.hamcrest.MatcherAssert;
import org.hamcrest.Matchers;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;

import java.util.ArrayList;
import java.util.List;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;


@RunWith(MockitoJUnitRunner.class)
public class AutonomousCommunityServiceImplTest {

    private final List<AutonomousCommunity> allCommunities = new ArrayList<>();
    private AutonomousCommunity firstCommunity;

    @InjectMocks
    private AutonomousCommunityServiceImpl service;

    @Mock
    private AutonomousCommunityRepository repository;

    @Before
    public void setUp() {
        firstCommunity = new AutonomousCommunity("AACC TS1");
        AutonomousCommunity secondCommunity = new AutonomousCommunity("AACC TS2");
        AutonomousCommunity thirdCommunity = new AutonomousCommunity("AACC TS3");
        firstCommunity.setId(1L);
        secondCommunity.setId(2L);
        thirdCommunity.setId(3L);
        allCommunities.add(firstCommunity);
        allCommunities.add(secondCommunity);
        allCommunities.add(thirdCommunity);
    }

    @Test
    public void findAllCommunitiesShouldReturnAllCommunities() {
        Mockito.when(repository.findAll()).thenReturn(allCommunities);
        List<AutonomousCommunity> currentCommunities = service.getAllCommunities(false);
        MatcherAssert.assertThat(allCommunities, Matchers.is(currentCommunities));
    }

    @Test
    public void findByNameShouldReturnThisCommunity() {
        Mockito.when(repository.findByName("AACC TS1")).thenReturn(firstCommunity);
        AutonomousCommunity communityFound = service.getByName("AACC TS1");
        assertThat(firstCommunity, is(communityFound));
    }
}
